from AccessControl        import getSecurityManager
from OFS.ObjectManager    import ObjectManager
from Globals              import DTMLFile
from SubObjectTypeSupport import SubObjectTypeSupport


class OrderedObjectManager(SubObjectTypeSupport, ObjectManager):
    ''' '''
    
    meta_type  ='Ordered Object Manager'

    __ac_permissions__ =  ( ( ('View', ('manage_move_object_up', 'manage_move_object_down',
                                        'manage_move_object_to_top', 'manage_move_object_to_bottom')), )
                            + SubObjectTypeSupport.__ac_permissions__ )

    manage_main = DTMLFile('dtml/main', globals())
    

    def get_object_position(self, id):
        ids = map(lambda obj: obj['id'], self._objects)
        return ids.index(id)


    def move_object_up(self, id):
        objects = list(self._objects)
        index = self.get_object_position(id)
        if index == 0: return 0

        objects[index], objects[index - 1] = objects[index - 1], objects[index]
        self._objects = tuple(objects)
        return 1

    
    def move_object_down(self, id):
        objects = list(self._objects)
        index = self.get_object_position(id)
        if index == len(objects) - 1: return 0 

        objects[index], objects[index + 1] = objects[index + 1], objects[index]
        self._objects = tuple(objects)
        return 1


    def move_object_to_top(self, id):
        objects = list(self._objects)
        index = self.get_object_position(id)
        if index == 0: return 0

        object = objects[index]
        objects.remove(object)
        objects.insert(0, object)
        self._objects = tuple(objects)
        return 1


    def move_object_to_bottom(self, id):
        objects = list(self._objects)
        index = self.get_object_position(id)
        if index == len(objects) - 1: return 0 

        object = objects[index]
        objects.remove(object)
        objects.append(object)
        self._objects = tuple(objects)
        return 1


    def manage_move_object_up(self, id, REQUEST=None):
        ''' '''
        self.move_object_up(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_down(self, id, REQUEST=None):
        ''' '''
        self.move_object_down(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_to_top(self, id, REQUEST=None):
        ''' '''
        self.move_object_to_top(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_to_bottom(self, id, REQUEST=None):
        ''' '''
        self.move_object_to_bottom(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def _setObject(self,id,object,roles=None,user=None, set_owner=1, position=None):
        ''' '''
        v=self._checkId(id)
        if v is not None: id=v
 
        try:    t=object.meta_type
        except: t=None

        for object_info in self._objects:
            if object_info['id'] == id:
                self._delObject(id)
                break

        # HERE IS MY PATCH
        if position is None: position = len(self._objects)
        objects = list(self._objects)
        objects.insert(position, {'id':id,'meta_type':t})
        self._objects = tuple(objects)
        
        if hasattr(object, '_p_jar') and object._p_jar is None:
            object._p_jar = self._p_jar
        self._setOb(id,object)
        object=self._getOb(id)

        if set_owner:
            object.manage_fixupOwnershipAfterAdd()

            if hasattr(object, '__ac_local_roles__'):
                if object.__ac_local_roles__ is None:
                    user=getSecurityManager().getUser()
                    name=user.getUserName()
                    if name != 'Anonymous User':
                        object.manage_setLocalRoles(name, ['Owner'])

        object.manage_afterAdd(object, self)
        return id


    def _delObject(self, id, dp=1):
        object=self._getOb(id)
        try:
            object.manage_beforeDelete(object, self)
        except BeforeDeleteException, ob:
            raise
        except:
            LOG('Zope',ERROR,'manage_beforeDelete() threw',
                error=sys.exc_info())
            pass
        # HERE IS MY PATCH
        objects = list(self._objects)
        objects.remove(objects[self.get_object_position(id)])
        self._objects = tuple(objects)
        self._delOb(id)

        try:    object._v__object_deleted__ = 1
        except: pass
        
